DROP TABLE IF EXISTS DOCQUEUE;
DROP TABLE IF EXISTS LANGUAGES CASCADE;
DROP TABLE IF EXISTS HEADERS CASCADE;
DROP TABLE IF EXISTS COMPANY CASCADE;
DROP TABLE IF EXISTS NOTES CASCADE;
DROP TABLE IF EXISTS PRINTERS CASCADE;
DROP TABLE IF EXISTS PRODUCT_COUNTER CASCADE;
DROP TABLE IF EXISTS PRODUCT_COUNTER CASCADE;
DROP TABLE IF EXISTS DRIVERS CASCADE;
DROP TABLE IF EXISTS TABLES;
DROP TABLE IF EXISTS WAITERS CASCADE;
DROP TABLE IF EXISTS CUSTOMERS CASCADE;
DROP TABLE IF EXISTS SUPPLIERS CASCADE;
DROP TABLE IF EXISTS PROFILES CASCADE;
DROP TABLE IF EXISTS RECIPE_TYPES CASCADE;
DROP TABLE IF EXISTS COOKBOOK CASCADE;
DROP TABLE IF EXISTS RECIPE_INGREDIENTS;
DROP TABLE IF EXISTS RECIPE_VARIANTS;
DROP TABLE IF EXISTS PLACEMENTS CASCADE;
DROP TABLE IF EXISTS ORDERS CASCADE;
DROP TABLE IF EXISTS ORDER_VARIANTS;
DROP TABLE IF EXISTS FRIDGE;
DROP TABLE IF EXISTS WAREHOUSE;
DROP TABLE IF EXISTS PLACEMENTS_ARCHIVE CASCADE;
DROP TABLE IF EXISTS ORDERS_ARCHIVE CASCADE;
DROP TABLE IF EXISTS ORDER_VARIANTS_ARCHIVE;
DROP TABLE IF EXISTS DATA_EXPORT_QUERY;
DROP TABLE IF EXISTS WEIGHTS;
DROP TABLE IF EXISTS EMAIL_SENDER;
DROP TABLE IF EXISTS RESERVATIONS;
DROP TABLE IF EXISTS TABLE_STORY;

DROP TABLE IF EXISTS order_product_table CASCADE;
DROP TABLE IF EXISTS order_table CASCADE;
DROP TABLE IF EXISTS printer_table CASCADE;
DROP TABLE IF EXISTS product_table CASCADE;
DROP TABLE IF EXISTS product_type_table CASCADE;
DROP TABLE IF EXISTS tax_rate_table CASCADE;
DROP TABLE IF EXISTS printer_group_table CASCADE;
DROP TABLE IF EXISTS printer_table CASCADE;
DROP TABLE IF EXISTS printer_type_table CASCADE;
DROP TABLE IF EXISTS driver_table CASCADE;
DROP TABLE IF EXISTS contact_table CASCADE;
DROP TABLE IF EXISTS alignment_table CASCADE;
DROP TABLE IF EXISTS node_table CASCADE;

DROP AGGREGATE IF EXISTS harray_agg(anyelement);

DROP SEQUENCE IF EXISTS receipt_serial_number_seq;
DROP SEQUENCE IF EXISTS invoice_serial_number_seq;
DROP SEQUENCE IF EXISTS delivery_serial_number_seq;
DROP SEQUENCE IF EXISTS order_serial_number_seq;
DROP SEQUENCE IF EXISTS refresh_tables_view_seq;
DROP SEQUENCE IF EXISTS clients_number_seq;


DROP LANGUAGE 'plpgsql' CASCADE;


CREATE TABLE node_table (

	id 				SERIAL PRIMARY KEY,
	
	name			TEXT
					DEFAULT '',  
	
	dbname			TEXT
					DEFAULT '',  


	username		TEXT
					DEFAULT '',  


	password			TEXT
					DEFAULT '',  


	hostname		TEXT
					DEFAULT ''

);


CREATE TABLE alignment_table (

	id 				SERIAL PRIMARY KEY,
	
	tableName		TEXT
					DEFAULT '',  
	
	tableId			INT
					DEFAULT NULL,
	
	state			TEXT
					DEFAULT 'new',	

	Timestamp	 	Timestamp
					DEFAULT CURRENT_TIMESTAMP,
					
	node			TEXT[]
					DEFAULT '{}'
					


);


CREATE TABLE contact_table (

	id 				SERIAL PRIMARY KEY,

	position			INT
					DEFAULT currval('contact_table_id_seq'),
	name  			TEXT
					DEFAULT ''
);


CREATE TABLE driver_table (

	id 				SERIAL PRIMARY KEY,

	position			INT
					DEFAULT currval('driver_table_id_seq'),
	name  			TEXT
					DEFAULT '',
	driver_name		TEXT
					DEFAULT '',
	serial_driver_name	TEXT
					DEFAULT '',
	lan_driver_name	TEXT
					DEFAULT ''
);


CREATE TABLE printer_type_table (

	id 				SERIAL PRIMARY KEY,
	position			INT,
	name  			TEXT
					DEFAULT ''

);


CREATE TABLE printer_table (

	id 			SERIAL PRIMARY KEY,
	
	position		INT
				DEFAULT currval('printer_table_id_seq'),
	
	name  		TEXT
				DEFAULT '',
	
	printer_type_id 	INT
					NOT NULL 
					REFERENCES printer_type_table(id) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,	

	ip_address	TEXT
				DEFAULT '',
	
	port			TEXT
				DEFAULT '',
	
	driver_table_id	INT
				NOT NULL 
				REFERENCES driver_table(id) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,
	
	show		TEXT
				DEFAULT 'true'
								
);


CREATE TABLE printer_group_table (

	id 				SERIAL PRIMARY KEY,
	position			INT,
	printer_table_id 	INT
					NOT NULL
					REFERENCES printer_table(id) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
					
	char_size			INT					

);


-- NUMERIC(4, 2)    DEFAULT 'NaN'     Not a Number

-- 4 cifre in tutto di cui 2 sono quelle decimali
-- Ci significa che tale numero sar compreso tra 00.00 e 99.99
-- NaN  un valore speciale equivalente al NULL per le stringhe

-- NUMERIC(3, 1)    

-- 3 cifre in tutto di cui 1 sola decimale e quindi 2 intere
-- Tale numero sar compreso tra 00.0 e 00.9


CREATE TABLE tax_rate_table (

	id 			SERIAL PRIMARY KEY,

	position		INT
				DEFAULT currval('tax_rate_table_id_seq'),

	name		TEXT
				DEFAULT '',
	
	fiscal_machine_department	
				INT
				DEFAULT 1,
	
	tax_rate		NUMERIC(4, 2)  -- 4 cifre in tutto di cui 2 decimali. Tale numero sar compreso tra 00.00 e 99.99 
				DEFAULT 0
				

);

CREATE TABLE product_type_table (

	id 			SERIAL PRIMARY KEY,
	position		INT
				DEFAULT currval('product_type_table_id_seq'),
	name		TEXT
				DEFAULT ''

);


CREATE TABLE product_table (

	id 					SERIAL PRIMARY KEY,
	position				INT
						DEFAULT currval('product_table_id_seq'),
	product_type_table_id	INT
						NOT NULL 
						REFERENCES product_type_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE,

	product_table_id		INT
						REFERENCES product_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE,

	name               		TEXT
						DEFAULT '',

	price				MONEY 
						DEFAULT '0',

	tax_rate_id			INT
						NOT NULL
						REFERENCES tax_rate_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE,

	price_with_tax			MONEY
						DEFAULT '0',
	
	discount_rate			NUMERIC(5, 2)  -- 5 cifre in tutto di cui 2 decimali. Tale numero sar compreso tra 000.00 e 999.99 
						DEFAULT 0, 

	printer_group_table_id 	INT
						NOT NULL
						REFERENCES printer_group_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE
);


CREATE TABLE order_table (

	id                  		SERIAL PRIMARY KEY,
	
	position			INT
					DEFAULT currval('order_table_id_seq'),
						
	name			TEXT
					DEFAULT '',
    
	serial_number		SERIAL,

	date			DATE
					DEFAULT CURRENT_DATE,
					
	contact_table_id		INT
						REFERENCES contact_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE

);


CREATE TABLE order_product_table (

	id 					SERIAL PRIMARY KEY,

	position				INT
						DEFAULT currval('order_product_table_id_seq'),
	
	quantity				INT
						DEFAULT 0,	
		
	order_table_id			INT
						REFERENCES order_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE,
						
	name               		TEXT
						DEFAULT '',

	order_product_table_id	INT
						REFERENCES order_product_table(id) 
						ON DELETE CASCADE
						ON UPDATE CASCADE,

	product_table_id		INT,
						-- REFERENCES product_table(id) 
						-- ON DELETE CASCADE
						-- ON UPDATE CASCADE,

	fiscal_machine_department	
						INT  
						DEFAULT 1, 

	price				MONEY
						DEFAULT '0',

	tax_rate				NUMERIC(5, 2)  -- 5 cifre in tutto di cui 2 decimali. Tale numero sar compreso tra 000.00 e 999.99 
						DEFAULT 0, 

	price_with_tax			MONEY
						DEFAULT '0',

	discount_rate			NUMERIC(5, 2)  -- 5 cifre in tutto di cui 2 decimali. Tale numero sar compreso tra 000.00 e 999.99 
						DEFAULT 0, 
						
	price_discounted		MONEY
						DEFAULT '0',

	price_discounted_with_tax 	
						MONEY
						DEFAULT '0',
	
	printed				TEXT
						DEFAULT 'no'
	
);



CREATE TABLE DOCQUEUE (

	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('docqueue_id_seq') + 1,
	MachineIpAddress TEXT,
	NumTable 	TEXT,
	PrinterName 	TEXT, 
	ConvertCommand TEXT,
	IconvCommand  TEXT,	
	PrintCommand   TEXT,	
	IpAddress TEXT,
	Port TEXT,
	Filename TEXT ,
	CopyNumber 	INT,
	JobStart	     	Time
				DEFAULT CURRENT_TIME,
	Printed TEXT 	CHECK( Printed IN  ('False', 'True'))
				DEFAULT 'False'
	
	
);


CREATE TABLE LANGUAGES (
	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('languages_id_seq') + 1,
	Language	TEXT,
	File			TEXT,
	HelpUrl		TEXT,
	AdvertisingUrl	TEXT

);



CREATE TABLE HEADERS (
	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('headers_id_seq') + 1,
	Text  		TEXT
);


CREATE TABLE COMPANY (

	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('company_id_seq') + 1,
	Type			TEXT,
	Header  		TEXT,
	BillingAddress	TEXT,
	ShippingAddress TEXT

);


CREATE TABLE NOTES (
	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('notes_id_seq') + 1,
	Note  		TEXT
);


CREATE TABLE DRIVERS (
	ID 			SERIAL PRIMARY KEY,
	Name  		TEXT,
	DriverName	TEXT
				DEFAULT('EpsonTM-T88'),
	SerialDriverName	TEXT
				DEFAULT('EpsonTM-T88Serial'),
	LanDriverName	TEXT
				DEFAULT('EpsonTM-T88Lan')

);

CREATE TABLE PRINTERS (
	ID 			SERIAL PRIMARY KEY,
	Name  		TEXT,
	Type 		TEXT CHECK( Type IN  ('Order', 'PreFiscal', 'Fiscal', 'Receipt', 'Invoice', 'Delivery', 'None')),
	IPAddress	TEXT,
	Port			TEXT,
	CharSize		INT,
	idDriver		INT
				NOT NULL 
				REFERENCES DRIVERS(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,
	Show		TEXT CHECK( Show IN  ('False', 'True'))
				DEFAULT 'True'
				
	
);


CREATE TABLE PRODUCT_COUNTER (

	ID 			SERIAL PRIMARY KEY,
	Name  		TEXT
				DEFAULT '',
	Show		TEXT 
				DEFAULT '0'
	
);


CREATE TABLE TABLES (
	ID 			SERIAL PRIMARY KEY,
	Number  		TEXT UNIQUE,
	NumChairs  	INT,       
	Status 		TEXT CHECK( Status IN  ('Occupied', 'Free', 'NotPaid', 'Reserved')),	
	Note                 TEXT,
	X			 INT,
	Y			 INT,
	IconFile		TEXT 
				DEFAULT ''
);



CREATE TABLE PROFILES (
	ID 			SERIAL PRIMARY KEY,
	Position		INT
				DEFAULT nextval('profiles_id_seq') + 1,
	Name		TEXT,
	ButtonPrice	INT,
	TablesView	INT,
	Orders		INT,
	Tables		INT,
	Waiters		INT,
	Profiles		INT,
	Customers	INT,
	Suppliers		INT,
	Tickets		INT,
	Menu		INT,
	Quantities	INT,
	Refrigerator	INT,
	StockHouse	INT,
	Notes		INT,
	Company		INT,
	Archive		INT,
	Printers		INT,
	Backup		INT,
	Preferences	INT,
	ButtonDelTable INT	
	
		
);



CREATE TABLE WAITERS (
	ID 			SERIAL PRIMARY KEY,
	Name               TEXT,
	Address            TEXT,
	Telephone		 TEXT,
	Cellular            TEXT,
	Note                 TEXT,
	Password		TEXT
				DEFAULT '',
	IDProfile		INT
				NOT NULL 
				REFERENCES PROFILES(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE

);


CREATE TABLE CUSTOMERS (
	ID 			SERIAL PRIMARY KEY,
	Name 		TEXT,
	BillingAddress 	TEXT,
	DeliveryAddress TEXT,
	Telephone	TEXT,
	Mail			TEXT,
	LicenseNumber TEXT,
        FiscalCode    	TEXT,
	Credit		REAL,
	Note 		TEXT,
	Barcode		TEXT  UNIQUE
				DEFAULT nextval('customers_id_seq') + 1,
	Timestamp 	Timestamp
				DEFAULT CURRENT_TIMESTAMP

);


CREATE TABLE SUPPLIERS (
	ID 			SERIAL PRIMARY KEY,
	Name 		TEXT,
	Address         	TEXT,
	Telephone	TEXT,
	Mail			TEXT,
	LicenseNumber TEXT,
        FiscalCode    	TEXT,
	Credit		REAL,
	BANK		TEXT,
	IBAN			TEXT,
	BIC			TEXT,
	Note 		TEXT
	
);


CREATE TABLE RECIPE_TYPES (
	ID 			SERIAL PRIMARY KEY,
	Position		INT
				DEFAULT nextval('recipe_types_id_seq') + 1,
	Name		TEXT,
	Department 	INT
				DEFAULT 1,
	TaxRate 		REAL
				DEFAULT 0


);	


CREATE TABLE COOKBOOK (
	ID 			SERIAL PRIMARY KEY,
	IDType		INT
				NOT NULL 
				REFERENCES RECIPE_TYPES(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	Name              TEXT,
	Price              	REAL,
	Visible		TEXT CHECK( Visible IN  ('No', 'Yes')),		
	IDPrinter 		INT
				NOT NULL
				REFERENCES PRINTERS(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,
	ButtonColor	TEXT
				DEFAULT 'lightGray',
	ButtonIcon	TEXT,
	Barcode		TEXT,
	AvailableQuantity  	REAL
					DEFAULT 0,
	ShortageQuantity     REAL
					DEFAULT 0,
	ReorderQuantity 	REAL 
					DEFAULT 0,
	PurchasePrice          REAL,
	Supplier			INT
					REFERENCES SUPPLIERS(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,

	PrepareTime		REAL
					DEFAULT 1,

	ProductCounterId	INT
					REFERENCES PRODUCT_COUNTER(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,
					
	Capacity			REAL
					DEFAULT 0,

	Conversion		REAL
					DEFAULT 1,
					
	Score			REAL	DEFAULT 0

	
);


CREATE TABLE RECIPE_INGREDIENTS (
	ID 			SERIAL PRIMARY KEY,
	IDRecipe         INT
				NOT NULL 
				REFERENCES COOKBOOK(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	Name              	TEXT,
	Quantity    	REAL,

	State		TEXT	
				DEFAULT 'new'	
	
	
);

CREATE TABLE RECIPE_VARIANTS (
	ID 			SERIAL PRIMARY KEY,
	IDRecipe         INT
				NOT NULL 
				REFERENCES COOKBOOK(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	IDRecipeType   INT
				NOT NULL 
				REFERENCES RECIPE_TYPES(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,

	Name              TEXT,
	Price                REAL,
	ButtonColor	TEXT
				DEFAULT 'lightGray',
	ButtonIcon	TEXT,
	
	State		TEXT	
				DEFAULT 'new'	
	
	
);


CREATE TABLE PLACEMENTS (
	ID 			SERIAL PRIMARY KEY,
	NumTable	 	TEXT UNIQUE,	
	Customer	INT
				REFERENCES CUSTOMERS(ID) 
					ON UPDATE CASCADE,
	
	CustomerName TEXT
				 DEFAULT '',

	Waiter	 	INT
				REFERENCES WAITERS(ID) 
					ON UPDATE CASCADE,	

	Note	 	TEXT,
	BegTimestamp 	Timestamp
					DEFAULT CURRENT_TIMESTAMP,
	EndTimestamp 	Timestamp,
	Total		REAL,
	Working		TEXT
				DEFAULT 'False',
	JobStart		Time
				DEFAULT CURRENT_TIME,
	JobDuration	REAL
				DEFAULT 0,
	JobPrinted	TEXT 
				DEFAULT 'True'
	
);


CREATE TABLE ORDERS (
	ID 			SERIAL PRIMARY KEY,
	IDPlacement   INT
				NOT NULL 
				REFERENCES PLACEMENTS(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	Position		INT,
	Recipe      	   TEXT,
	Number    	   REAL,
	Price      	  	   REAL,
	IDPrinter 		INT
				NOT NULL
				REFERENCES PRINTERS(ID) 
					ON DELETE CASCADE				
					ON UPDATE CASCADE,
	Printed		TEXT CHECK( Printed IN  ('No', 'Yes')),
	IDRecipeType	INT
				NOT NULL 
				REFERENCES RECIPE_TYPES(ID) 
					ON DELETE CASCADE				
					ON UPDATE CASCADE
	

);


CREATE TABLE ORDER_VARIANTS (
	ID 			SERIAL PRIMARY KEY,
	IDOrder 		INT
				NOT NULL 
				REFERENCES ORDERS(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	Name      			 TEXT,
	Quantity    		 REAL,
	Price      	  		 REAL

);


CREATE TABLE FRIDGE (

	ID 			SERIAL PRIMARY KEY,
	Ingredient	TEXT UNIQUE,
	Quantity  		REAL,
	Capacity			REAL
					DEFAULT 0,
	Conversion		REAL
					DEFAULT 1


);	


CREATE TABLE WAREHOUSE (

	ID 			SERIAL PRIMARY KEY,
	Ingredient	TEXT UNIQUE,
	Quantity  		REAL
				DEFAULT 0,
	ShortageQuantity     REAL
					DEFAULT 0,
	ReorderQuantity 	REAL
					DEFAULT 0,
	PurchasePrice          REAL,
	Supplier			INT
					REFERENCES SUPPLIERS(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	Capacity			REAL
					DEFAULT 0,
	Conversion		REAL
					DEFAULT 1
					

);	


CREATE TABLE PLACEMENTS_ARCHIVE (
	ID 			SERIAL PRIMARY KEY,
	NumTable	 	TEXT,	
	Customer		TEXT,	
	Waiter		TEXT,
	Note	 	TEXT,
	BegTimestamp 	Timestamp
					DEFAULT CURRENT_TIMESTAMP,
	EndTimestamp 	Timestamp,
	Credit		REAL,
	UseCredit		TEXT CHECK( UseCredit IN  ('No', 'Yes')),
	FiscalDocType  TEXT,
	Total		REAL,
	Discount		REAL,
	TotalDiscounted REAL,
	Cash		REAL,
	CreditCard	REAL,
	Bancomat	REAL,
	Other		REAL,
	Paid			REAL,
	NotPaid		REAL,
        SerialNumber	TEXT,
        Taxable            REAL,
        TaxRate            REAL,
        Tax                   REAL
	
	
);


CREATE TABLE ORDERS_ARCHIVE (
	ID 			SERIAL PRIMARY KEY,
	IDPlacementArchive   	INT
						NOT NULL 
						REFERENCES PLACEMENTS_ARCHIVE(ID) 
							ON DELETE CASCADE
							ON UPDATE CASCADE,
	IDRecipeType  		 	INT
						NOT NULL 
						REFERENCES RECIPE_TYPES(ID) 
							ON DELETE CASCADE
							ON UPDATE CASCADE,
	Position		   INT,
	Recipe      	   TEXT,
	Number    	   INT,
	Price      	  	   REAL,
	IDPrinter 		INT
				NOT NULL
				REFERENCES PRINTERS(ID) 
					ON DELETE CASCADE				
					ON UPDATE CASCADE
	

);


CREATE TABLE ORDER_VARIANTS_ARCHIVE (
	ID 			SERIAL PRIMARY KEY,
	IDOrderArchive 		INT
						NOT NULL 
						REFERENCES ORDERS_ARCHIVE(ID) 
							ON DELETE CASCADE
							ON UPDATE CASCADE,
	
	Name      			 TEXT,
	Quantity    		 REAL,
	Price      	  		 REAL

);



CREATE TABLE DATA_EXPORT_QUERY (

	ID 			SERIAL PRIMARY KEY,
	Query		TEXT,
	FieldSeparatorCode  INT,
	OutputFileName TEXT

);


CREATE TABLE  WEIGHTS (

	ID 			SERIAL PRIMARY KEY,
	Tare			TEXT,
	Weight		TEXT,
	Price		TEXT,
	Amount		TEXT,
	Timestamp 	Timestamp
				DEFAULT CURRENT_TIMESTAMP

);


CREATE TABLE  EMAIL_SENDER (

	ID 			SERIAL PRIMARY KEY,
	Header		TEXT

);


CREATE TABLE RESERVATIONS (

	ID 			SERIAL PRIMARY KEY,

	Timestamp 	Timestamp
				DEFAULT CURRENT_TIMESTAMP,

	Type			TEXT,

	Date	 	Timestamp
				DEFAULT CURRENT_TIMESTAMP,

	Customer	INT
				REFERENCES CUSTOMERS(ID) 
					ON UPDATE CASCADE,

	CustomerName TEXT
				 DEFAULT '',
	
	Description	TEXT,
	
	Note		TEXT,

	Total		MONEY,
	
	Advance		MONEY,
	
	Balance		MONEY,	
	
	PaymentNote	TEXT
	

);


CREATE TABLE TABLE_STORY (

	ID 			SERIAL PRIMARY KEY,

	Timestamp 	Timestamp
				DEFAULT CURRENT_TIMESTAMP,

	NumTable 	TEXT,


	Waiter	 	INT
				REFERENCES WAITERS(ID) 
					ON UPDATE CASCADE,	

	Total 		REAL 
	
);


CREATE LANGUAGE 'plpgsql' HANDLER plpgsql_call_handler LANCOMPILER 'PL/pgSQL';


CREATE FUNCTION unsetTableStatus() RETURNS opaque AS '

	DECLARE 

	BEGIN

		update TABLES set Status = ''Free'' where Number = OLD.NumTable;

		return NEW;
	END;

' LANGUAGE 'plpgsql';



CREATE TRIGGER unsetTableStatusTrigger
			AFTER DELETE
			ON PLACEMENTS FOR EACH ROW
			EXECUTE PROCEDURE unsetTableStatus();



CREATE FUNCTION setTableStatus() RETURNS opaque AS '

	DECLARE 

	BEGIN

		update TABLES set Status = ''Occupied'' where Number = NEW.NumTable;

		return NEW;
	END;

' LANGUAGE 'plpgsql';



CREATE TRIGGER setTableStatusTrigger
			AFTER INSERT
			ON PLACEMENTS FOR EACH  ROW
			EXECUTE PROCEDURE setTableStatus();



CREATE FUNCTION updateTableStatus() RETURNS opaque AS '

	DECLARE 

		_value text;

	BEGIN

		select into _value (select status from Tables where number = OLD.NumTable);
		-- La seguente istruzione server per fare la stampa della variabile _value
         -- RAISE NOTICE ''_value: %'', _value;
         IF _value is NULL THEN
             _value := ''Occupied'';
         END IF;
		update TABLES set Status = ''Free'' where Number = OLD.NumTable;
		update TABLES set Status = _value where Number = NEW.NumTable;

		return NEW;
	END;

' LANGUAGE 'plpgsql';



CREATE TRIGGER updateTableStatusTrigger
			AFTER UPDATE
			ON PLACEMENTS FOR EACH  ROW
			EXECUTE PROCEDURE updateTableStatus();




CREATE FUNCTION recipe_typesInsert() RETURNS opaque AS '

	BEGIN
		insert into alignment_table (tablename, tableid, state) values(''recipe_types'', new.id, ''new'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_typesInsertTrigger
			AFTER INSERT
			ON recipe_types FOR EACH ROW
			EXECUTE PROCEDURE recipe_typesInsert();


CREATE FUNCTION recipe_typesUpdate() RETURNS opaque AS '

	BEGIN

		insert into alignment_table (tablename, tableid, state) values(''recipe_types'', new.id, ''change'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_typesUpdateTrigger
			AFTER UPDATE
			ON recipe_types FOR EACH ROW
			EXECUTE PROCEDURE recipe_typesUpdate();


CREATE FUNCTION recipe_typesDelete() RETURNS opaque AS '

	BEGIN
		-- RAISE NOTICE ''old.id: %'', old.id;
		insert into alignment_table (tablename, tableid, state) values(''recipe_types'', old.id, ''deleted'');
		return old;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_typesDeleteTrigger
			AFTER DELETE
			ON recipe_types FOR EACH ROW
			EXECUTE PROCEDURE recipe_typesDelete();



CREATE FUNCTION cookbookInsert() RETURNS opaque AS '

	BEGIN
		insert into alignment_table (tablename, tableid, state) values(''cookbook'', new.id, ''new'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER cookbookInsertTrigger
			AFTER INSERT
			ON cookbook FOR EACH ROW
			EXECUTE PROCEDURE cookbookInsert();


CREATE FUNCTION cookbookUpdate() RETURNS opaque AS '

	BEGIN

		insert into alignment_table (tablename, tableid, state) values(''cookbook'', new.id, ''change'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER cookbookUpdateTrigger
			AFTER UPDATE
			ON cookbook FOR EACH ROW
			EXECUTE PROCEDURE cookbookUpdate();


CREATE FUNCTION cookbookDelete() RETURNS opaque AS '

	BEGIN
		-- RAISE NOTICE ''old.id: %'', old.id;
		insert into alignment_table (tablename, tableid, state) values(''cookbook'', old.id, ''deleted'');
		return old;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER cookbookDeleteTrigger
			AFTER DELETE
			ON cookbook FOR EACH ROW
			EXECUTE PROCEDURE cookbookDelete();


CREATE FUNCTION recipe_variantsInsert() RETURNS opaque AS '

	BEGIN
		insert into alignment_table (tablename, tableid, state) values(''recipe_variants'', new.id, ''new'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_variantsInsertTrigger
			AFTER INSERT
			ON recipe_variants FOR EACH ROW
			EXECUTE PROCEDURE recipe_variantsInsert();


CREATE FUNCTION recipe_variantsUpdate() RETURNS opaque AS '

	BEGIN
		insert into alignment_table (tablename, tableid, state) values(''recipe_variants'', new.id, ''change'');		
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_variantsUpdateTrigger
			AFTER UPDATE
			ON recipe_variants FOR EACH ROW
			EXECUTE PROCEDURE recipe_variantsUpdate();


CREATE FUNCTION recipe_variantsDelete() RETURNS opaque AS '

	BEGIN
		-- RAISE NOTICE ''old.id: %'', old.id;
		insert into alignment_table (tablename, tableid, state) values(''recipe_variants'', old.id, ''deleted'');
		return old;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_variantsDeleteTrigger
			AFTER DELETE
			ON recipe_variants FOR EACH ROW
			EXECUTE PROCEDURE recipe_variantsDelete();



CREATE FUNCTION recipe_ingredientsInsert() RETURNS opaque AS '

	BEGIN
		insert into alignment_table (tablename, tableid, state) values(''recipe_ingredients'', new.id, ''new'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_ingredientsInsertTrigger
			AFTER INSERT
			ON recipe_ingredients FOR EACH ROW
			EXECUTE PROCEDURE recipe_ingredientsInsert();


CREATE FUNCTION recipe_ingredientsUpdate() RETURNS opaque AS '

	BEGIN
		insert into alignment_table (tablename, tableid, state) values(''recipe_ingredients'', new.id, ''change'');
		return new;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_ingredientsUpdateTrigger
			AFTER UPDATE
			ON recipe_ingredients FOR EACH ROW
			EXECUTE PROCEDURE recipe_ingredientsUpdate();


CREATE FUNCTION recipe_ingredientsDelete() RETURNS opaque AS '

	BEGIN
		-- RAISE NOTICE ''old.id: %'', old.id;
		insert into alignment_table (tablename, tableid, state) values(''recipe_ingredients'', old.id, ''deleted'');
		return old;
	END;

' LANGUAGE 'plpgsql';

CREATE TRIGGER recipe_ingredientsDeleteTrigger
			AFTER DELETE
			ON recipe_ingredients FOR EACH ROW
			EXECUTE PROCEDURE recipe_ingredientsDelete();




CREATE FUNCTION toint(text) RETURNS integer AS '

        DECLARE 

              _strvalue ALIAS FOR $1;
			_value integer;

        BEGIN
				IF _strvalue = '''' THEN
					RETURN 0;
				ELSE
					 SELECT INTO _value CAST(_strvalue AS INT);

					RETURN _value;
				END IF;

        END;

' LANGUAGE 'plpgsql';


CREATE FUNCTION getVariantTotal(int) RETURNS real AS '

        DECLARE 

              _id ALIAS FOR $1;
			_result real;

        BEGIN

			SELECT INTO _result (SELECT sum(order_variants_archive.price * order_variants_archive.quantity * orders_archive.number) as price from order_variants_archive, orders_archive where order_variants_archive.idorderarchive = orders_archive.id and order_variants_archive.idorderarchive = _id);
			
			IF _result != 0 THEN
				RETURN _result;
			ELSE
				RETURN 0;
			END IF;
			

        END;

' LANGUAGE 'plpgsql';




CREATE FUNCTION getPlacementTotal(int) RETURNS real AS '

        DECLARE 

              _id ALIAS FOR $1;
			_result real;

        BEGIN

			SELECT INTO _result (SELECT sum((orders.price * orders.number) + getPlacementTotalVariant(orders.id)) from orders where idplacement = _id);
			
			IF _result != 0 THEN
				RETURN _result;
			ELSE
				RETURN 0;
			END IF;
			

        END;

' LANGUAGE 'plpgsql';



CREATE FUNCTION getPlacementTotalVariant(int) RETURNS real AS '

        DECLARE 

              _id ALIAS FOR $1;
			_result real;

        BEGIN

			SELECT INTO _result (SELECT sum(order_variants.price * order_variants.quantity * orders.number) as price from order_variants, orders where order_variants.idorder = orders.id and order_variants.idorder = _id);
			
			IF _result != 0 THEN
				RETURN _result;
			ELSE
				RETURN 0;
			END IF;
			

        END;

' LANGUAGE 'plpgsql';

CREATE AGGREGATE harray_agg(anyelement) (
SFUNC=array_append,
STYPE=anyarray,
INITCOND='{}' );

CREATE SEQUENCE receipt_serial_number_seq MINVALUE 0;
SELECT nextval('receipt_serial_number_seq');

CREATE SEQUENCE invoice_serial_number_seq MINVALUE 0;
SELECT nextval('invoice_serial_number_seq');

CREATE SEQUENCE delivery_serial_number_seq MINVALUE 0;
SELECT nextval('delivery_serial_number_seq');

CREATE SEQUENCE order_serial_number_seq MINVALUE 0;
SELECT nextval('order_serial_number_seq');

CREATE SEQUENCE refresh_tables_view_seq MINVALUE 0;
SELECT nextval('refresh_tables_view_seq');

CREATE SEQUENCE clients_number_seq MINVALUE 0;
SELECT nextval('clients_number_seq');


